###################################################
### chunk number 1: version
###################################################
#line 21 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
prettyVersion <- packageDescription("RcppEigen")$Version
prettyDate <- format(Sys.Date(), "%B %e, %Y")


###################################################
### chunk number 2: preliminaries
###################################################
#line 29 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
link <- function( f, package, text = f, root = "http://finzi.psych.upenn.edu/R/library/" ){
	h <- if( missing(package) ) {
		as.character( help( f ) )
	} else {
		as.character( help( f, package = paste( package, sep = "" ) ) )
	}
	if( ! length(h) ){
		sprintf( "\\\\textbf{%s}", f )
	} else {
		rx <- "^.*/([^/]*?)/help/(.*?)$"
		package <- sub( rx, "\\1", h, perl = TRUE )
		page <- sub( rx, "\\2", h, perl = TRUE )
		sprintf( "\\\\href{%s%s/html/%s.html}{\\\\texttt{%s}}", root, package, page, text )
	}
}
linkS4class <- function( cl, package, text = cl, root = "http://finzi.psych.upenn.edu/R/library/" ){
	link( sprintf("%s-class", cl), package, text, root )
}
require( inline )
require( RcppEigen )


###################################################
### chunk number 3: Adef
###################################################
#line 219 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
(A <- matrix(1:6, ncol=2))
str(A)


###################################################
### chunk number 4: transCpp
###################################################
#line 230 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
transCpp <-'
using Eigen::Map;
using Eigen::MatrixXi;
                 // Map the integer matrix AA from R
const Map<MatrixXi>  A(as<Map<MatrixXi> >(AA));
                 // evaluate and return the transpose of A
const MatrixXi      At(A.transpose());
return wrap(At);
'


###################################################
### chunk number 5: transCppLst
###################################################
#line 242 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
cat(transCpp, "\n")


###################################################
### chunk number 6: ftrans
###################################################
#line 246 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
ftrans <- cxxfunction(signature(AA="matrix"), transCpp, plugin="RcppEigen")
(At <- ftrans(A))
stopifnot(all.equal(At, t(A)))


###################################################
### chunk number 7: prodCpp
###################################################
#line 265 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
prodCpp <- '
using Eigen::Map;
using Eigen::MatrixXi;
const Map<MatrixXi>    B(as<Map<MatrixXi> >(BB));
const Map<MatrixXi>    C(as<Map<MatrixXi> >(CC));
return List::create(_["B %*% C"]         = B * C,
                    _["crossprod(B, C)"] = B.adjoint() * C);
'


###################################################
### chunk number 8: prodCppLst
###################################################
#line 276 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
cat(prodCpp, "\n")


###################################################
### chunk number 9: prod
###################################################
#line 280 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
fprod <- cxxfunction(signature(BB = "matrix", CC = "matrix"), prodCpp, "RcppEigen")
B <- matrix(1:4, ncol=2); C <- matrix(6:1, nrow=2)
str(fp <- fprod(B, C))
stopifnot(all.equal(fp[[1]], B %*% C), all.equal(fp[[2]], crossprod(B, C)))


###################################################
### chunk number 10:  eval=FALSE
###################################################
## #line 298 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
## t(X) %*% X


###################################################
### chunk number 11: crossprod
###################################################
#line 325 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
crossprodCpp <- '
using Eigen::Map;
using Eigen::MatrixXi;
using Eigen::Lower;

const Map<MatrixXi> A(as<Map<MatrixXi> >(AA));
const int           m(A.rows()), n(A.cols());                        
MatrixXi          AtA(MatrixXi(n, n).setZero().
                      selfadjointView<Lower>().rankUpdate(A.adjoint()));
MatrixXi          AAt(MatrixXi(m, m).setZero().
                      selfadjointView<Lower>().rankUpdate(A));

return List::create(_["crossprod(A)"]  = AtA,
                    _["tcrossprod(A)"] = AAt);
'


###################################################
### chunk number 12: crossprodCppLst
###################################################
#line 343 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
cat(crossprodCpp, "\n")


###################################################
### chunk number 13: 
###################################################
#line 347 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
fcprd <- cxxfunction(signature(AA = "matrix"), crossprodCpp, "RcppEigen")
str(crp <- fcprd(A))
stopifnot(all.equal(crp[[1]], crossprod(A)), all.equal(crp[[2]], tcrossprod(A)))


###################################################
### chunk number 14: storage
###################################################
#line 388 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
storage.mode(A) <- "double"


###################################################
### chunk number 15: cholCpp
###################################################
#line 392 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
cholCpp <- '
using Eigen::Map;
using Eigen::MatrixXd;
using Eigen::LLT;
using Eigen::Lower;

const Map<MatrixXd>   A(as<Map<MatrixXd> >(AA));
const int             n(A.cols());
const LLT<MatrixXd> llt(MatrixXd(n, n).setZero().
                        selfadjointView<Lower>().rankUpdate(A.adjoint()));

return List::create(_["L"] = MatrixXd(llt.matrixL()),
                    _["R"] = MatrixXd(llt.matrixU()));
'


###################################################
### chunk number 16: cholCppLst
###################################################
#line 409 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
cat(cholCpp, "\n")


###################################################
### chunk number 17: fchol
###################################################
#line 413 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
fchol <- cxxfunction(signature(AA = "matrix"), cholCpp, "RcppEigen")
(ll <- fchol(A))
stopifnot(all.equal(ll[[2]], chol(crossprod(A))))


###################################################
### chunk number 18: 
###################################################
#line 441 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
cholDetCpp <- '
using Eigen::Lower;
using Eigen::Map;
using Eigen::MatrixXd;
using Eigen::VectorXd;

const Map<MatrixXd>   A(as<Map<MatrixXd> >(AA));
const int             n(A.cols());
const MatrixXd      AtA(MatrixXd(n, n).setZero().
                        selfadjointView<Lower>().rankUpdate(A.adjoint()));
const MatrixXd     Lmat(AtA.llt().matrixL());
const double       detL(Lmat.diagonal().prod());
const VectorXd     Dvec(AtA.ldlt().vectorD());

return List::create(_["d1"] = detL * detL,
                    _["d2"] = Dvec.prod(),
                    _["ld"] = Dvec.array().log().sum());
'


###################################################
### chunk number 19: cholDetCppLst
###################################################
#line 462 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
cat(cholDetCpp, "\n")


###################################################
### chunk number 20: fdet
###################################################
#line 466 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
fdet <- cxxfunction(signature(AA = "matrix"), cholDetCpp, "RcppEigen")
unlist(ll <- fdet(A))


###################################################
### chunk number 21: lltLSCpp
###################################################
#line 504 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
lltLSCpp <- '
using Eigen::LLT;
using Eigen::Lower;
using Eigen::Map;
using Eigen::MatrixXd;
using Eigen::VectorXd;

const Map<MatrixXd>   X(as<Map<MatrixXd> >(XX));
const Map<VectorXd>   y(as<Map<VectorXd> >(yy));
const int             n(X.rows()), p(X.cols());
const LLT<MatrixXd> llt(MatrixXd(p, p).setZero().
                        selfadjointView<Lower>().rankUpdate(X.adjoint()));
const VectorXd  betahat(llt.solve(X.adjoint() * y));
const VectorXd   fitted(X * betahat);
const VectorXd    resid(y - fitted);
const int            df(n - p);
const double          s(resid.norm() / std::sqrt(double(df)));
const VectorXd       se(s * llt.matrixL().solve(MatrixXd::Identity(p, p)).
                        colwise().norm());
return     List::create(_["coefficients"]   = betahat,
                        _["fitted.values"]  = fitted,
                        _["residuals"]      = resid,
                        _["s"]              = s,
                        _["df.residual"]    = df,
                        _["rank"]           = p,
                        _["Std. Error"]     = se);
'


###################################################
### chunk number 22: lltLSCppLst
###################################################
#line 534 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
cat(lltLSCpp, "\n")


###################################################
### chunk number 23: lltLS
###################################################
#line 545 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
lltLS <- cxxfunction(signature(XX = "matrix", yy = "numeric"), lltLSCpp, "RcppEigen")
data(trees, package="datasets")
str(lltFit <- with(trees, lltLS(cbind(1, log(Girth)), log(Volume))))
str(lmFit <- with(trees, lm.fit(cbind(1, log(Girth)), log(Volume))))
for (nm in c("coefficients", "residuals", "fitted.values", "rank"))
    stopifnot(all.equal(lltFit[[nm]], unname(lmFit[[nm]])))
stopifnot(all.equal(lltFit[["Std. Error"]],
                    unname(coef(summary(lm(log(Volume) ~ log(Girth), trees)))[,2])))


###################################################
### chunk number 24: missingcell
###################################################
#line 644 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
dd <- data.frame(f1 = gl(4, 6, labels = LETTERS[1:4]),
                 f2 = gl(3, 2, labels = letters[1:3]))[-(7:8), ]
xtabs(~ f2 + f1, dd)                    # one missing cell
mm <- model.matrix(~ f1 * f2, dd)
kappa(mm)         # large condition number, indicating rank deficiency
rcond(mm)         # alternative evaluation, the reciprocal condition number
(c(rank=qr(mm)$rank, p=ncol(mm))) # rank as computed in R's qr function
set.seed(1)
dd$y <- mm %*% seq_len(ncol(mm)) + rnorm(nrow(mm), sd = 0.1)
                         # lm detects the rank deficiency
fm1 <- lm(y ~ f1 * f2, dd)
writeLines(capture.output(print(summary(fm1), signif.stars=FALSE))[9:22])


###################################################
### chunk number 25: rankdeficientPQR
###################################################
#line 841 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
print(summary(fmPQR <- fastLm(y ~ f1 * f2, dd)), signif.stars=FALSE)
all.equal(coef(fm1), coef(fmPQR))
all.equal(unname(fitted(fm1)), fitted(fmPQR))
all.equal(unname(residuals(fm1)), residuals(fmPQR))


###################################################
### chunk number 26: rankdeficientSVD
###################################################
#line 850 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
print(summary(fmSVD <- fastLm(y ~ f1 * f2, dd, method=4L)), signif.stars=FALSE)
all.equal(coef(fm1), coef(fmSVD))
all.equal(unname(fitted(fm1)), fitted(fmSVD))
all.equal(unname(residuals(fm1)), residuals(fmSVD))


###################################################
### chunk number 27: rankdeficientVLV
###################################################
#line 857 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
print(summary(fmVLV <- fastLm(y ~ f1 * f2, dd, method=5L)), signif.stars=FALSE)
all.equal(coef(fmSVD), coef(fmVLV))
all.equal(unname(fitted(fm1)), fitted(fmSVD))
all.equal(unname(residuals(fm1)), residuals(fmSVD))


###################################################
### chunk number 28: benchmark eval=FALSE
###################################################
## #line 880 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
## source(system.file("examples", "lmBenchmark.R", package="RcppEigen"))


###################################################
### chunk number 29: badtransCpp
###################################################
#line 962 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
badtransCpp <- '
using Eigen::Map;
using Eigen::MatrixXi;
const Map<MatrixXi>  A(as<Map<MatrixXi> >(AA));
return wrap(A.transpose());
'


###################################################
### chunk number 30: badtransCppLst
###################################################
#line 971 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
cat(badtransCpp, "\n")


###################################################
### chunk number 31: ftrans2
###################################################
#line 975 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
Ai <- matrix(1:6, ncol=2L)
ftrans2 <- cxxfunction(signature(AA = "matrix"), badtransCpp, "RcppEigen")
(At <- ftrans2(Ai))
all.equal(At, t(Ai))


###################################################
### chunk number 32: 
###################################################
#line 991 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
sparseProdCpp <- '
using Eigen::Map;
using Eigen::MappedSparseMatrix;
using Eigen::SparseMatrix;
using Eigen::VectorXd;

const MappedSparseMatrix<double>  A(as<MappedSparseMatrix<double> >(AA));
const Map<VectorXd>               y(as<Map<VectorXd> >(yy));
const SparseMatrix<double>       At(A.adjoint());
return List::create(_["At"]  = At,
                    _["Aty"] = At * y);
'


###################################################
### chunk number 33: sparseProdCppLst
###################################################
#line 1007 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
cat(sparseProdCpp, "\n")


###################################################
### chunk number 34: 
###################################################
#line 1011 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
sparse1 <- cxxfunction(signature(AA = "dgCMatrix", yy = "numeric"),
                       sparseProdCpp, "RcppEigen")
data(KNex, package="Matrix")
rr <- sparse1(KNex$mm, KNex$y)
stopifnot(all.equal(rr$At, t(KNex$mm)),
          all.equal(rr$Aty, as.vector(crossprod(KNex$mm, KNex$y))))


###################################################
### chunk number 35:  eval=FALSE
###################################################
## #line 1033 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
## code <- '
## using Eigen::Map;
## using Eigen::MatrixXcd;
##   // Map the complex matrix A_ from R
## const Map<MatrixXcd>    A(as<Map<MatrixXcd> >(A_));
## return List::create(_["transpose"] = A.transpose(),
##                     _["adjoint"]   = A.adjoint());
## '
## writeLines( code, "code.cpp" )


###################################################
### chunk number 36:  eval=FALSE
###################################################
## #line 1044 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
## ex_highlight( "code.cpp" )


###################################################
### chunk number 37:  eval=FALSE
###################################################
## #line 1048 "d:/Rcompile/CRANpkg/local/2.12/RcppEigen/inst/doc/RcppEigen-Intro.Rnw"
## fadj <-
##     cxxfunction(signature( A_ = "matrix"),
##                 paste(readLines( "code.cpp" ), collapse = "\n"),
##                 plugin = "RcppEigen")
## A <- matrix(1:6 + 1i*(6:1), nc=2)
## fadj(A)


